import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:flutter_mylibrary/base/bloc/bloc_provider.dart.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';



abstract class SmartRefreshBlocBase<T> extends PullToRefreshBloc<T> {

  bool requestSuccess = false;

  EasyRefreshController easyRefreshController = EasyRefreshController();


  @override
  void dispose() {
    // TODO: implement dispose
  }




  /// 添加空布局
  Widget buildEmptyWidget(String message, {Widget emptyWidget}){
    return emptyWidget == null? Container(
      alignment: Alignment.center,
      child: Text(
          message
      ),
    ) : emptyWidget;
  }

  Widget buildFirstRefreshWidget(){
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        new CircularProgressIndicator(),
        Gaps.vGap10,
        new Text(
          '加载中',
          style: TextStyle(color: mainTextColor),
        )
      ],
    );
  }


  ///自定义头部
  Header headerCreate({BuildContext context});

  ///自定义脚部
  Footer footerCreate({BuildContext context});


  int page = 0;

  List<T> data = [];


  ///自定义第一页页数
  int firstPage();

  ///自定义一页数量
  int pageSize();

  bool hasMore = true;

  void setHasMore(bool hasMore){
    this.hasMore = hasMore;
  }


  EasyRefresh getEasyRefresh(Widget widget, {dataRefresh, dataOnLoad, Widget emptyWidget, bool firstRefresh = true, Widget firstRefreshWidget, bool openLoadMore = true, BuildContext context}){
    return EasyRefresh(
      header: headerCreate(context: context),
      footer: footerCreate(context: context),
      controller: easyRefreshController,
      enableControlFinishLoad: true,
      enableControlFinishRefresh: true,
      child: widget,
      onRefresh: ()async{
        page = firstPage();
        data.clear();
        setHasMore(true);
        onRefresh(page, pageSize()).then((value) {
          easyRefreshController.finishRefresh();
          easyRefreshController.resetLoadState();
          emptyWidget = null;
          data.addAll(value);
          eventBus.fire(data);
          page++;
        });
      },
      onLoad: openLoadMore? ()async{
        onLoadMore(page, pageSize()).then((value) {
          List<T> newData = value;
          data.addAll(newData);
          easyRefreshController.finishLoad(noMore: !hasMore);
          page++;
          eventBus.fire(data);
        });
      }: null,
      emptyWidget: requestSuccess? null : emptyWidget,
      firstRefreshWidget: firstRefresh?  buildFirstRefreshWidget() : null,
      firstRefresh: firstRefresh,
    );

  }



  EasyRefresh getEasyRefreshCustom(List<Widget> slivers,  EasyRefreshController easyRefreshController, {dataRefresh, dataOnLoad, Widget emptyWidget, bool firstRefresh = true, bool openLoadMore = true, BuildContext context}){
    return EasyRefresh.custom(
      header: headerCreate(context: context),
      footer: footerCreate(context: context),
      controller: easyRefreshController,
      enableControlFinishLoad: true,
      enableControlFinishRefresh: true,
      slivers: slivers,
      onRefresh: ()async{
        page = firstPage();
        onRefresh(page, pageSize()).then((value) {
          easyRefreshController.finishRefresh();
          easyRefreshController.resetLoadState();
          emptyWidget = null;
          data.addAll(value);
          eventBus.fire(data);
        });
      },
      onLoad: openLoadMore? ()async{
        onLoadMore(page, pageSize()).then((value) {
          List<T> newData = value;
          data.addAll(newData);
          easyRefreshController.finishLoad(noMore: newData.length <= pageSize());
          page++;
          eventBus.fire(data);
        });
      }: null,
      emptyWidget: requestSuccess? null : emptyWidget,
      firstRefreshWidget: firstRefresh?  buildFirstRefreshWidget() : null,
      firstRefresh: firstRefresh,

    );
  }




}


///下拉刷新Bloc
abstract class PullToRefreshBloc<T> extends BlocBase {
  ///加载数据
  Future<List<T>> getData(int page, int pageSize);

  ///刷新
  Future<List<T>> onRefresh(int page, int pageSize);

  ///更多
  Future<List<T>> onLoadMore(int page, int pageSize);
}
