import 'package:flutter/cupertino.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:flutter_mylibrary/base/provider/request_info.dart';
import 'package:flutter_mylibrary/http/dio_utils.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';
import '../view_state_model.dart';

abstract class ViewStateRequestModel<T> extends ViewStateModel{

  T data;

  RequestInfo getRequestInfo();

  void initData() async{
    RequestInfo requestInfo = getRequestInfo();
    data = await DioUtils.getInstance().request(
      requestInfo?.method,
      requestInfo?.url,
      data: requestInfo?.data,
      onFinish: (){
        refreshController.finishRefresh();
        refreshController.resetRefreshState();
      },
      onError: (e){
        setError(e, e.message, message: e.message);
      }
    ).then((res) => requestInfo?.handleRes(res.data));
    if (!isError) {
      setIdle();
    }
    if (isIdle && isEmpty(data)) {
      ///请求无异常但是数据为空需要设置为empty
      setEmpty();
    }
  }


  ///获取一个公共的刷新控件
  Widget getEasyRefreshWidget({Widget child, bool shrinkWrap = false, Widget emptyWidget, List<Widget> slivers}){
    return getViewStateWidget(this, () => EasyRefresh.custom(
      emptyWidget: emptyWidget??getRefreshNoDataWidget(viewStateModel: this),
      slivers: slivers?? [
        SliverToBoxAdapter(
          child: child,
        )
      ],
      shrinkWrap: shrinkWrap,
      onRefresh: isBusy? null : () async{
        initData();
      },
      controller: refreshController,
    ));
  }
}

