import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';

import 'view_state_list_model.dart';

/// 基于
abstract class ViewStateRefreshListModel<T> extends ViewStateListModel<T> {

  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;



  ///获取一个公共的刷新控件
  Widget getEasyRefreshWidget({Widget child, bool shrinkWrap = false, Widget emptyWidget, List<Widget> slivers}){
    return getViewStateWidget(this, () => EasyRefresh.custom(
      emptyWidget: emptyWidget??getRefreshNoDataWidget(viewStateModel: this),
      slivers: isIdle? slivers??[
        SliverToBoxAdapter(
          child: child,
        )
      ] : null,
      shrinkWrap: shrinkWrap,
      onRefresh: isBusy? null : () async{
        refresh();
      },
      onLoad: (isBusy || !hasMore)? null : () async{
        loadMore();
      },
      controller: refreshController,
    ));
  }


  /// 当前页码
  int _currentPageNum;

  /// 下拉刷新
  ///
  /// [init] 是否是第一次加载
  /// true:  Error时,需要跳转页面
  /// false: Error时,不需要跳转页面,直接给出提示
  Future<List<T>> refresh({bool init = false}) async {
    hasMore = true;
    try {
      _currentPageNum = firstPage();
      var data = await loadData(pageNum: firstPage());
      if (data.isEmpty) {
        hasMore = false;
        refreshController.resetRefreshState();
        list.clear();
        setEmpty();
      } else {
        onCompleted(data);
        list.clear();
        list.addAll(data);
        // 小于分页的数量,禁止上拉加载更多
        if (data.length < pageSize()) {
          hasMore = false;
          refreshController.finishRefresh();
          refreshController.finishLoad();
          refreshController.resetRefreshState();
        } else {
          //防止上次上拉加载更多失败,需要重置状态
          refreshController.finishRefresh();
          refreshController.resetRefreshState();
          refreshController.resetLoadState();
        }
        setIdle();
      }
      return data;
    } catch (e, s) {
      /// 页面已经加载了数据,如果刷新报错,不应该直接跳转错误页面
      /// 而是显示之前的页面数据.给出错误提示
      if (init){
        list.clear();
        setError(e, s);
      }
      hasMore = false;
      refreshController.finishRefresh();
      refreshController.finishLoad();
      return list;
    }
  }

  /// 上拉加载更多
  Future<List<T>> loadMore() async {
    try {
      if (hasMore) {
        var data = await loadData(pageNum: ++_currentPageNum);
        if (data.isEmpty) {
          _currentPageNum--;
          hasMore = false;
          refreshController.finishLoad();
        } else {
          onCompleted(data);
          list.addAll(data);
          if (data.length < pageSize()) {
            hasMore = false;
            refreshController.finishLoad();
          } else {
            hasMore = true;
            refreshController.finishLoad();
          }
          notifyListeners();
        }
        return data;
      }else{
        ///没有更多了
        refreshController.finishLoad(noMore: true);
        return list;
      }
    } catch (e, s) {
      _currentPageNum--;
      refreshController.finishLoad(success: false, noMore: true);
      debugPrint('error--->\n' + e.toString());
      debugPrint('statck--->\n' + s.toString());
      return list;
    }
  }



  bool _hasMore;

  ///设置是否还有更多
  set hasMore(bool hasMore){
    this._hasMore = hasMore;
  }

  bool get hasMore => _hasMore??true;

  // 加载数据
  Future<List<T>> loadData({int pageNum});

  @override
  void dispose() {
    refreshController.dispose();
    super.dispose();
  }


}
